/**
 *
 * WapiUtil.java
 *
 * Copyright 2012 Infoblox Inc. All rights reserved.
 *             
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright 
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY INFOBLOX AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL INFOBLOX OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of Infoblox.
 *
 */
package com.infoblox.wapi.common;

import java.io.FileInputStream;
import java.security.KeyStore;
import java.security.cert.X509Certificate;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;
import javax.net.ssl.TrustManagerFactory;
import javax.net.ssl.X509TrustManager;

import org.apache.log4j.Logger;

/**
 * Utility methods, including certificate operations
 */
public class WapiUtil {

    /** Log4j logger */
    static private Logger logger = Logger.getLogger(WapiUtil.class);

    /**
     * Gets the TrustManagers for server authentication
     * 
     * @param sessionConfig
     * @return TrustManager[]
     */
    public static TrustManager[] getTrustManagers(WapiConfig sessionConfig) {

        try {
            KeyStore keystore = KeyStore.getInstance("JKS"); //$NON-NLS-1$
            keystore.load(new FileInputStream(sessionConfig.getTrustFile()),
                    sessionConfig.getTrustFilePass().toCharArray());
            TrustManagerFactory factory = TrustManagerFactory
                    .getInstance("SunX509"); //$NON-NLS-1$
            factory.init(keystore);
            TrustManager trustManagers[] = factory.getTrustManagers();
            return trustManagers;
        } catch (Exception e) {
            logger.error("Exception occurred while obtaining Trust Managers", e); //$NON-NLS-1$
            throw new RuntimeException(e);
        }
    }

    /**
     * Gets a TrustManager that will allow authentication from any server
     * 
     * @return TrustManager[]
     */
    public static TrustManager[] getTrustAllManagers() {

        X509TrustManager trustAllManager = new X509TrustManager() {
            /**
             * @param chain
             * @param authType
             */
            @Override
            public void checkClientTrusted(X509Certificate[] chain,
                    String authType) {
                // Empty by design.
            }

            /**
             * @param chain
             * @param authType
             */
            @Override
            public void checkServerTrusted(X509Certificate[] chain,
                    String authType) {
                // Empty by design.
            }

            @Override
            public X509Certificate[] getAcceptedIssuers() {
                return (new X509Certificate[] {});
            }
        };
        return new TrustManager[] { trustAllManager };
    }

    /**
     * Enable a trust manager that does not validate certificate chains.
     * 
     * @param trustCerts
     *            TrustManager[] with trust certificates.
     */
    public static void trustHttpsCertificates(TrustManager[] trustCerts) {

        javax.net.ssl.SSLContext sc;
        try {
            sc = javax.net.ssl.SSLContext.getInstance("SSL"); //$NON-NLS-1$

            sc.init(null, trustCerts, null);
            javax.net.ssl.HttpsURLConnection.setDefaultSSLSocketFactory(sc
                    .getSocketFactory());
        } catch (Exception e) {
            logger.error("Exception occurred while setup of https settings", e); //$NON-NLS-1$
            throw new RuntimeException(e);
        }
    }

    /**
     * Enable a host name verifier that does not validate host names at all.
     */
    public static void trustAllHostNames() {
        HostnameVerifier hv = new HostnameVerifier() {
            /**
             * @param arg0
             * @param arg1
             */
            @Override
            public boolean verify(String arg0, SSLSession arg1) {
                return true;
            }
        };
        HttpsURLConnection.setDefaultHostnameVerifier(hv);
    }
}
