/**
 *
 * XMLConverter.java
 *
 * Copyright 2012 Infoblox Inc. All rights reserved.
 *             
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright 
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY INFOBLOX AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL INFOBLOX OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of Infoblox.
 *
 */

package com.infoblox.wapi.converter;

import java.io.StringReader;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;

import com.sun.org.apache.xerces.internal.parsers.DOMParser;

/**
 * WAPI converter for XML objects.
 */
public class XMLConverter extends AbstractConverter<Document> {
    /** Log4j logger */
    static private Logger logger = Logger.getLogger(XMLConverter.class);

    /**
     * @param newString
     * @return Document
     * @see com.infoblox.wapi.converter.AbstractConverter#stringToObject(java.lang.String)
     */
    @Override
    public Document stringToObject(String newString) {
        DOMParser parser = new DOMParser();
        try {
            InputSource source = new InputSource(new StringReader(newString));
            parser.parse(source);
        } catch (Exception e) {
            logger.error("An exception occurred when parsing XML", e); //$NON-NLS-1$
            throw new RuntimeException(e);
        }
        return parser.getDocument();
    }

    /**
     * @param newObject
     * @return String
     * @see com.infoblox.wapi.converter.AbstractConverter#objectToString(java.lang.Object)
     */
    @Override
    public String objectToString(Document newObject) {
        return documentToString(newObject);
    }

}
