/**
 *
 * OperationType.java
 *
 * Copyright 2012 Infoblox Inc. All rights reserved.
 *             
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright 
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY INFOBLOX AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL INFOBLOX OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of Infoblox.
 *
 */

package com.infoblox.wapi.java.client.sample;

import com.infoblox.wapi.dao.IWapiDAO;

/**
 * Operation Type enumeration with all Web API operations like Create, Delete,
 * Update, etc.
 */
@SuppressWarnings({ "rawtypes", "unchecked" })
public enum OperationType {

    /** Create operation. */
    CREATE("create") { //$NON-NLS-1$
        @Override
        public String performOperation(IWapiDAO dao, String params) {
            return "Created with ref: " //$NON-NLS-1$
                    + dao.create(dao.getInputConverter().stringToObject(params));
        }
    },
    /** Read all objects operation. */
    READ_ALL("readall") { //$NON-NLS-1$
        /**
         * @param params
         */
        @Override
        public String performOperation(IWapiDAO dao, String params) {
            return "Objects found: " + dao.getOutputConverter().objectToString(dao.readAll()); //$NON-NLS-1$
        }
    },
    /** Read specific object operation. */
    READ("read") { //$NON-NLS-1$
        /**
         * @param params
         */
        @Override
        public String performOperation(IWapiDAO dao, String params) {
            return "Object read: " + dao.getOutputConverter().objectToString(dao.read()); //$NON-NLS-1$
        }
    },
    /** Object update operation. */
    UPDATE("update") { //$NON-NLS-1$
        @Override
        public String performOperation(IWapiDAO dao, String params) {
            return "Updated with ref: " + dao.update(dao.getInputConverter().stringToObject(params)); //$NON-NLS-1$
        }
    },
    /** Search objects operation. */
    SEARCH("search") { //$NON-NLS-1$
        @Override
        public String performOperation(IWapiDAO dao, String params) {
            return "Objects found: " //$NON-NLS-1$
                    + dao.getOutputConverter().objectToString(
                            dao.search(dao.getInputConverter().stringToObject(
                                    params)));
        }
    },
    /** Object delete operation. */
    DELETE("delete") { //$NON-NLS-1$
        /**
         * @param params
         */
        @Override
        public String performOperation(IWapiDAO dao, String params) {
            return "Deleted with ref: " + dao.delete(); //$NON-NLS-1$
        }
    };

    /** String with type value. */
    private String typeValue;

    /**
     * @param operation
     */
    private OperationType(String operation) {
        typeValue = operation;
    }

    /**
     * Get Operation by input String.
     * 
     * @param operationType
     * @return String
     */
    static public OperationType getOperation(String operationType) {
        for (OperationType type : OperationType.values()) {
            if (type.getTypeValue().equals(operationType.toLowerCase())) {
                return type;
            }
        }
        throw new RuntimeException("Unknown operation: " + operationType); //$NON-NLS-1$
    }

    /**
     * @return String
     */
    public String getTypeValue() {
        return typeValue;
    }

    /**
     * @param dao
     * @param params
     * @return String
     */

    public abstract String performOperation(IWapiDAO dao, String params);
}
